@@ -33,32 +33,15 @@
             RTCOM_TYPE_EL_ITER, RTComElIterPrivate))
 G_DEFINE_TYPE(RTComElIter, rtcom_el_iter, G_TYPE_OBJECT);
 
-typedef struct _RTComElIterRepresentation RTComElIterRepresentation;
-struct _RTComElIterRepresentation {
-    /* Here go values which are likely to get used soon,
-     * for fast lookup. Of course they refer to the db
-     * row the iter points to _right now_. */
-    guint id;
-    guint service_id;
-
-    /* These two are taken from the INNER JOIN. We need them handy. */
-    gchar * service;
-    gchar * event_type;
-
-    RTComElPlugin * plugin;
-};
-
 typedef struct _RTComElIterPrivate RTComElIterPrivate;
 struct _RTComElIterPrivate {
     RTComEl * el;
     RTComElQuery * query;
 
-    sqlite3 * db;
-    sqlite3_stmt * stmt;
-    gchar * sql; /* The sql used to create this iterator. Used to refresh. */
+    rtcom_el_db_t db;
+    rtcom_el_db_stmt_t stmt;
 
     GHashTable * plugins;
-    RTComElIterRepresentation * rep;
 
     /* Row data, as returned by rtocm_el_query_get_row() */
     GHashTable * columns;
@@ -66,6 +49,13 @@ struct _RTComElIterPrivate {
     /* Whether this iterator is atomic and should close the
      * transaction when getting disposed. */
     gboolean atomic;
+
+    /* Current values (valid only when iterator points at a result row). */
+    gint current_event_id;
+    gint current_service_id;
+    gint current_event_type_id;
+    RTComElPlugin *currently_active_plugin;
+
 };
 
 enum
@@ -74,7 +64,6 @@ enum
     RTCOM_EL_ITER_PROP_QUERY,
     RTCOM_EL_ITER_PROP_DB,
     RTCOM_EL_ITER_PROP_STMT,
-    RTCOM_EL_ITER_PROP_SQL,
     RTCOM_EL_ITER_PROP_PLUGINS,
     RTCOM_EL_ITER_PROP_ATOMIC,
 };
@@ -85,7 +74,6 @@ void _update_representation(
     g_assert(priv);
 
     g_return_if_fail(priv->stmt);
-    g_return_if_fail(priv->rep);
 
     if (priv->columns)
         rtcom_el_db_schema_update_row (priv->stmt, priv->columns);
@@ -93,23 +81,15 @@ void _update_representation(
         priv->columns = rtcom_el_db_schema_get_row (priv->stmt);
 
 #define LOOKUP_INT(x) (g_value_get_int (g_hash_table_lookup (priv->columns, x)))
-#define LOOKUP_STR(x) (g_value_dup_string (g_hash_table_lookup (priv->columns, x)))
 
-    priv->rep->id = LOOKUP_INT("id");
-    priv->rep->service_id = LOOKUP_INT("service-id");
+    priv->current_event_id = LOOKUP_INT("id");
+    priv->current_service_id = LOOKUP_INT("service-id");
+    priv->current_event_type_id = LOOKUP_INT("event-type-id");
 
-    priv->rep->plugin = g_hash_table_lookup(
-            priv->plugins,
-            GINT_TO_POINTER(priv->rep->service_id));
-
-    g_free(priv->rep->service);
-    g_free(priv->rep->event_type);
-
-    priv->rep->service = LOOKUP_STR("service");
-    priv->rep->event_type = LOOKUP_STR("event-type");
+    priv->currently_active_plugin = g_hash_table_lookup (priv->plugins,
+        GINT_TO_POINTER (priv->current_service_id));
 
 #undef LOOKUP_INT
-#undef LOOKUP_STR
 }
 
 static gboolean _find_value(
@@ -127,8 +107,8 @@ static gboolean _find_value(
     priv = RTCOM_EL_ITER_GET_PRIV(it);
 
     /* Ask the plugin */
-    if(priv->rep->plugin && priv->rep->plugin->get_value)
-        got_value = priv->rep->plugin->get_value(it, key, value);
+    if(priv->currently_active_plugin && priv->currently_active_plugin->get_value)
+        got_value = priv->currently_active_plugin->get_value(it, key, value);
 
     if(!got_value)
     {
@@ -177,10 +157,6 @@ static void rtcom_el_iter_set_property(
             priv->stmt = g_value_get_pointer(value);
             break;
 
-        case RTCOM_EL_ITER_PROP_SQL:
-            priv->sql = g_value_dup_string(value);
-            break;
-
         case RTCOM_EL_ITER_PROP_PLUGINS:
             priv->plugins = g_value_get_pointer(value);
             break;
@@ -221,10 +197,6 @@ static void rtcom_el_iter_get_property(
             g_value_set_pointer(value, priv->stmt);
             break;
 
-        case RTCOM_EL_ITER_PROP_SQL:
-            g_value_set_string(value, priv->sql);
-            break;
-
         case RTCOM_EL_ITER_PROP_PLUGINS:
             g_value_set_pointer(value, priv->plugins);
             break;
@@ -249,14 +221,13 @@ static void rtcom_el_iter_init(
     priv->query = NULL;
     priv->db = NULL;
     priv->stmt = NULL;
-    priv->sql = NULL;
     priv->plugins = NULL;
     priv->atomic = FALSE;
 
-    /* FIXME: we don't really need this, as we already have everything
-     * cached in priv->columns.  */
-    priv->rep =
-        (RTComElIterRepresentation *) g_new0(RTComElIterRepresentation, 1);
+    priv->current_event_id = -1;
+    priv->current_service_id = -1;
+    priv->current_event_type_id = -1;
+    priv->currently_active_plugin = NULL;
 }
 
 static GObject * rtcom_el_iter_constructor(
@@ -299,12 +270,6 @@ static void rtcom_el_iter_finalize(
         priv->columns = NULL;
     }
 
-    g_free(priv->sql);
-
-    g_free(priv->rep->service);
-    g_free(priv->rep->event_type);
-    g_free(priv->rep);
-
     G_OBJECT_CLASS(rtcom_el_iter_parent_class)->finalize(object);
 }
 
@@ -354,16 +319,6 @@ static void rtcom_el_iter_class_init(
                 "The SQLite3 Statement for this iterator",
                 G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
-    g_object_class_install_property(
-            object_class,
-            RTCOM_EL_ITER_PROP_SQL,
-            g_param_spec_string(
-                "sql",
-                "SQL string",
-                "The SQL used to create this iterator",
-                "", /* default */
-                G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-
     g_object_class_install_property(
             object_class,
             RTCOM_EL_ITER_PROP_PLUGINS,
@@ -433,59 +388,6 @@ gboolean rtcom_el_iter_next(
     return TRUE;
 }
 
-gboolean rtcom_el_iter_refresh(
-        RTComElIter * it)
-{
-    RTComElIterPrivate * priv = NULL;
-    gint status;
-
-    g_return_val_if_fail(it, FALSE);
-    g_return_val_if_fail(RTCOM_IS_EL_ITER(it), FALSE);
-
-    priv = RTCOM_EL_ITER_GET_PRIV(it);
-    g_return_val_if_fail(priv->db, FALSE);
-    g_return_val_if_fail(priv->sql, FALSE);
-
-    if (priv->stmt != NULL)
-    {
-        sqlite3_finalize(priv->stmt);
-        priv->stmt = NULL;
-    }
-
-    if(sqlite3_prepare(
-                priv->db, priv->sql, -1, &(priv->stmt), NULL) != SQLITE_OK)
-    {
-        g_warning("Could not compile: '%s': %s.",
-                priv->sql, sqlite3_errmsg(priv->db));
-        if(priv->stmt)
-        {
-            sqlite3_finalize(priv->stmt);
-            priv->stmt = NULL;
-        }
-    }
-    else
-    {
-        status = rtcom_el_db_iterate (priv->db, priv->stmt, NULL);
-        if(status == SQLITE_DONE)
-        {
-            sqlite3_finalize(priv->stmt);
-            priv->stmt = NULL;
-            return FALSE;
-        }
-        if(status != SQLITE_ROW)
-        {
-            g_warning("Could not step statement: %s",
-                    sqlite3_errmsg(priv->db));
-            sqlite3_finalize(priv->stmt);
-            priv->stmt = NULL;
-            return FALSE;
-        }
-
-    }
-
-    return rtcom_el_iter_first(it);;
-}
-
 GValueArray * rtcom_el_iter_get_valuearray(
         RTComElIter * it,
         ...)
@@ -501,9 +403,9 @@ GValueArray * rtcom_el_iter_get_valuearray(
     priv = RTCOM_EL_ITER_GET_PRIV(it);
     g_return_val_if_fail(priv->db, NULL);
     g_return_val_if_fail(priv->columns, NULL);
-    g_return_val_if_fail(priv->rep, NULL);
-    g_return_val_if_fail(priv->rep->service, NULL);
-    g_return_val_if_fail(priv->rep->event_type, NULL);
+
+    g_warning ("%s: deprecated, use rtcom_el_iter_get_value_map() "
+        "or rtcom_el_iter_get_values() instead", G_STRFUNC);
 
     va_start(ap, it);
     if(!(item = va_arg(ap, gchar *)))
@@ -525,11 +427,131 @@ GValueArray * rtcom_el_iter_get_valuearray(
     return ret;
 }
 
+GHashTable *
+rtcom_el_iter_get_value_map (RTComElIter * it, ...)
+{
+    RTComElIterPrivate * priv = NULL;
+    va_list ap;
+    gchar * item = NULL;
+    GHashTable *map = NULL;
+
+    g_return_val_if_fail(RTCOM_IS_EL_ITER(it), NULL);
+
+    priv = RTCOM_EL_ITER_GET_PRIV(it);
+    g_return_val_if_fail(priv->db, NULL);
+    g_return_val_if_fail(priv->columns, NULL);
+
+    va_start(ap, it);
+    if(!(item = va_arg(ap, gchar *)))
+    {
+        va_end(ap);
+        return NULL;
+    }
+
+    map = g_hash_table_new_full (g_str_hash, g_str_equal, g_free,
+      rtcom_el_db_g_value_slice_free);
+
+    while(item)
+    {
+        GValue *val = g_slice_new0 (GValue);
+
+        _find_value(it, item, val);
+        g_hash_table_insert (map, g_strdup (item), val);
+
+        item = va_arg(ap, gchar *);
+    }
+    va_end(ap);
+
+    return map;
+}
+
+gboolean
+rtcom_el_iter_get_values (RTComElIter * it, ...)
+{
+    RTComElIterPrivate * priv = NULL;
+    va_list ap;
+    gchar * item = NULL;
+
+    g_return_val_if_fail(RTCOM_IS_EL_ITER(it), FALSE);
+
+    priv = RTCOM_EL_ITER_GET_PRIV(it);
+    g_return_val_if_fail(priv->db, FALSE);
+    g_return_val_if_fail(priv->columns, FALSE);
+
+    va_start(ap, it);
+    if(!(item = va_arg(ap, gchar *)))
+      {
+        va_end(ap);
+        return FALSE;
+      }
+
+    while(item)
+      {
+        GValue *val = g_slice_new0 (GValue);
+
+        if (!_find_value(it, item, val))
+          {
+            va_end (ap);
+            g_slice_free (GValue, val);
+            return FALSE;
+          }
+
+        switch (G_VALUE_TYPE (val))
+          {
+            case G_TYPE_INT:
+              {
+                gint *x = va_arg (ap, gint *);
+
+                *x = g_value_get_int (val);
+                break;
+              }
+
+            case G_TYPE_BOOLEAN:
+              {
+                gboolean *x = va_arg (ap, gboolean *);
+
+                *x = g_value_get_boolean (val);
+                break;
+              }
+
+            case G_TYPE_STRING:
+              {
+                gchar **x = va_arg (ap, gchar **);
+
+                *x = g_value_dup_string (val);
+                break;
+              }
+
+            default:
+              {
+                g_error ("%s: bug in library, unexpected type %s", G_STRFUNC,
+                    G_VALUE_TYPE_NAME (val));
+
+                va_end (ap);
+
+                g_value_unset(val);
+                g_slice_free (GValue, val);
+
+                return FALSE;
+              }
+          }
+
+        g_value_unset (val);
+        g_slice_free (GValue, val);
+
+        item = va_arg(ap, gchar *);
+      }
+
+    va_end(ap);
+
+    return TRUE;
+}
+
 RTComElAttachIter * rtcom_el_iter_get_attachments(
         RTComElIter * it)
 {
     RTComElIterPrivate * priv = NULL;
-    sqlite3_stmt * stmt = NULL;
+    rtcom_el_db_stmt_t stmt = NULL;
     gint status;
 
     g_return_val_if_fail(RTCOM_IS_EL_ITER(it), NULL);
@@ -537,10 +559,10 @@ RTComElAttachIter * rtcom_el_iter_get_attachments(
     priv = RTCOM_EL_ITER_GET_PRIV(it);
     g_return_val_if_fail(priv->db, NULL);
     g_return_val_if_fail(priv->stmt, NULL);
-    g_return_val_if_fail(priv->rep, NULL);
 
-    /* XXX: should this query be part of this iterator's transaction,
-     *      to avoid potential inconsistencies with concurrent access? */
+    /* Note: to avoid potential inconsistencies, the caller should call
+     * rtcom_el_get_events_atomic() which wraps the whole query and all
+     * subselects inside a transaction. */
 
     if (sqlite3_prepare_v2 (priv->db,
                             "SELECT id, event_id, path, desc"
@@ -551,7 +573,7 @@ RTComElAttachIter * rtcom_el_iter_get_attachments(
         goto ret_null;
     }
 
-    if (sqlite3_bind_int (stmt, 1, priv->rep->id) != SQLITE_OK)
+    if (sqlite3_bind_int (stmt, 1, priv->current_event_id) != SQLITE_OK)
     {
         g_warning ("could not bind the event ID parameter: %s", sqlite3_errmsg(priv->db));
         goto ret_null;
@@ -584,24 +606,9 @@ gboolean rtcom_el_iter_get_int(
         const gchar * key,
         gint * ret)
 {
-    GValue value = {0};
-
-    g_return_val_if_fail(RTCOM_IS_EL_ITER(it), FALSE);
-    g_return_val_if_fail(ret != NULL, FALSE);
-
-    if(!_find_value(it, key, &value))
-        return FALSE;
-
-    if(!G_VALUE_HOLDS_INT(&value))
-    {
-        g_value_unset(&value);
-        return FALSE;
-    }
-
-    *ret = g_value_get_int(&value);
-    g_value_unset(&value);
-
-    return TRUE;
+    g_warning ("%s: deprecated, use rtcom_el_iter_get_values() instead",
+        G_STRFUNC);
+    return rtcom_el_iter_get_values (it, key, ret, NULL);
 }
 
 gboolean rtcom_el_iter_dup_string(
@@ -609,27 +616,9 @@ gboolean rtcom_el_iter_dup_string(
         const gchar * key,
         gchar ** ret)
 {
-    GValue value = {0};
-
-    g_return_val_if_fail(RTCOM_IS_EL_ITER(it), FALSE);
-    g_return_val_if_fail(ret != NULL, FALSE);
-
-    if(!_find_value(it, key, &value))
-    {
-        g_warning("%s: couldn't find value '%s'.", G_STRLOC, key);
-        return FALSE;
-    }
-
-    if(!G_VALUE_HOLDS_STRING(&value))
-    {
-        g_value_unset(&value);
-        return FALSE;
-    }
-
-    *ret = g_value_dup_string(&value);
-    g_value_unset(&value);
-
-    return TRUE;
+    g_warning ("%s: deprecated, use rtcom_el_iter_get_values() instead",
+        G_STRFUNC);
+    return rtcom_el_iter_get_values (it, key, ret, NULL);
 }
 
 /* Plugins functions */
@@ -661,7 +650,35 @@ gboolean rtcom_el_iter_get_raw(
     return TRUE;
 }
 
-gboolean rtcom_el_iter_get(
+const GHashTable *
+rtcom_el_iter_get_columns (RTComElIter *it)
+{
+  RTComElIterPrivate * priv = NULL;
+
+  g_return_val_if_fail(RTCOM_IS_EL_ITER(it), NULL);
+
+  priv = RTCOM_EL_ITER_GET_PRIV(it);
+  g_return_val_if_fail(priv->stmt, NULL);
+
+  if (priv->columns == NULL)
+    {
+      g_debug ("%s: No results received, returning nothing", G_STRFUNC);
+      return NULL;
+    }
+
+  return priv->columns;
+}
+
+gboolean
+rtcom_el_iter_get (RTComElIter *it, RTComElEvent *ev)
+{
+  g_warning ("%s: deprecated, please use rtcom_el_iter_get_columns() "
+      "or rtcom_el_iter_get_full() instead", G_STRFUNC);
+
+  return rtcom_el_iter_get_full (it, ev);
+}
+
+gboolean rtcom_el_iter_get_full (
         RTComElIter * it,
         RTComElEvent * ev)
 {
@@ -688,6 +705,7 @@ gboolean rtcom_el_iter_get(
     RTCOM_EL_EVENT_SET_FIELD(ev, start_time,       g_value_get_int(g_hash_table_lookup(priv->columns, "start-time")));
     RTCOM_EL_EVENT_SET_FIELD(ev, end_time,         g_value_get_int(g_hash_table_lookup(priv->columns, "end-time")));
     RTCOM_EL_EVENT_SET_FIELD(ev, is_read,          g_value_get_boolean(g_hash_table_lookup(priv->columns, "is-read")));
+    RTCOM_EL_EVENT_SET_FIELD(ev, outgoing,         g_value_get_boolean(g_hash_table_lookup(priv->columns, "outgoing")));
     RTCOM_EL_EVENT_SET_FIELD(ev, flags,            g_value_get_int(g_hash_table_lookup(priv->columns, "flags")));
     RTCOM_EL_EVENT_SET_FIELD(ev, bytes_sent,       g_value_get_int(g_hash_table_lookup(priv->columns, "bytes-sent")));
     RTCOM_EL_EVENT_SET_FIELD(ev, bytes_received,   g_value_get_int(g_hash_table_lookup(priv->columns, "bytes-received")));
@@ -712,11 +730,11 @@ gboolean rtcom_el_iter_get(
     RTCOM_EL_EVENT_SET_FIELD(ev, pango_markup,     (gchar *) g_value_dup_string(g_hash_table_lookup(priv->columns, "service")));
 
     /* FIXME: this will look better when we ditch the Event struct fields */
-    if(priv->rep->plugin && priv->rep->plugin->get_value)
+    if(priv->currently_active_plugin && priv->currently_active_plugin->get_value)
     {
         GValue tmp = {0,};
         gboolean ret =
-            priv->rep->plugin->get_value(it, "additional-text", &tmp);
+            priv->currently_active_plugin->get_value(it, "additional-text", &tmp);
         if (ret)
         {
             g_free (RTCOM_EL_EVENT_GET_FIELD (ev, additional_text));
@@ -725,7 +743,7 @@ gboolean rtcom_el_iter_get(
             g_value_unset(&tmp);
         }
 
-        ret = priv->rep->plugin->get_value(it, "icon-name", &tmp);
+        ret = priv->currently_active_plugin->get_value(it, "icon-name", &tmp);
         if (ret)
         {
             g_free (RTCOM_EL_EVENT_GET_FIELD (ev, icon_name));
@@ -734,7 +752,7 @@ gboolean rtcom_el_iter_get(
             g_value_unset(&tmp);
         }
 
-        ret = priv->rep->plugin->get_value(it, "pango-markup", &tmp);
+        ret = priv->currently_active_plugin->get_value(it, "pango-markup", &tmp);
         if (ret)
         {
             g_free (RTCOM_EL_EVENT_GET_FIELD (ev, pango_markup));
@@ -761,11 +779,10 @@ gchar * rtcom_el_iter_get_header_raw(
 
     priv = RTCOM_EL_ITER_GET_PRIV(it);
     g_return_val_if_fail(priv->db, NULL);
-    g_return_val_if_fail(priv->rep, NULL);
 
     sql = sqlite3_mprintf(
             "SELECT value from Headers where event_id = %d and name = %Q;",
-            priv->rep->id,
+            priv->current_event_id,
             key);
     if(sqlite3_prepare(priv->db, sql, -1, &stmt, NULL) != SQLITE_OK)
     {
@@ -793,31 +810,5 @@ gchar * rtcom_el_iter_get_header_raw(
     return ret;
 }
 
-const gchar * rtcom_el_iter_get_service(
-        RTComElIter * it)
-{
-    RTComElIterPrivate * priv = NULL;
-
-    g_return_val_if_fail(RTCOM_IS_EL_ITER(it), NULL);
-
-    priv = RTCOM_EL_ITER_GET_PRIV(it);
-    g_return_val_if_fail(priv->rep, NULL);
-
-    return priv->rep->service;
-}
-
-const gchar * rtcom_el_iter_get_event_type(
-        RTComElIter * it)
-{
-    RTComElIterPrivate * priv = NULL;
-
-    g_return_val_if_fail(RTCOM_IS_EL_ITER(it), NULL);
-
-    priv = RTCOM_EL_ITER_GET_PRIV(it);
-    g_return_val_if_fail(priv->rep, NULL);
-
-    return priv->rep->event_type;
-}
-
 /* vim: set ai et tw=75 ts=4 sw=4: */
 
